import { createReducer, on } from '@ngrx/store';
import {
  initialState,
  adapterReceiveList,
  adapterSendList,
  adapterReservationList,
  adapterSearchList
} from './state';
import * as AuthenticationStore from '@app/store/account/authentication';
import {
  retrieveMessageSuccess,
  retrieveMessage,
  searchMessage,
  searchMessageSuccess
} from './actions';
import { MessageType } from '@ucap-webmessenger/api-message';

export const reducer = createReducer(
  initialState,

  on(retrieveMessage, (state, action) => {
    switch (action.messageType) {
      case MessageType.Receive: {
        return {
          ...state,
          receiveListProcessing: true
        };
      }
      case MessageType.Send: {
        return {
          ...state,
          sendListProcessing: true
        };
      }
      case MessageType.Reservation: {
        return {
          ...state,
          reservationListProcessing: true
        };
      }
    }
  }),
  on(retrieveMessageSuccess, (state, action) => {
    switch (action.messageType) {
      case MessageType.Receive: {
        return {
          ...state,
          receiveList: adapterReceiveList.upsertMany(action.res.messageList, {
            ...state.receiveList
          }),
          receiveListProcessing: false
        };
      }
      case MessageType.Send: {
        return {
          ...state,
          sendList: adapterSendList.upsertMany(action.res.messageList, {
            ...state.sendList
          }),
          sendListProcessing: false
        };
      }
      case MessageType.Reservation: {
        return {
          ...state,
          reservationList: adapterReservationList.upsertMany(
            action.res.messageList,
            {
              ...state.reservationList
            }
          ),
          reservationListProcessing: false
        };
      }
    }
  }),

  on(searchMessage, (state, action) => {
    return {
      ...state,
      searchList: adapterSearchList.removeAll(state.searchList),
      searchListProcessing: true
    };
  }),
  on(searchMessageSuccess, (state, action) => {
    return {
      ...state,
      searchList: adapterSearchList.upsertMany(action.res.messageList, {
        ...state.searchList
      }),
      searchListProcessing: false
    };
  }),

  on(AuthenticationStore.logoutInitialize, (state, action) => {
    return {
      ...initialState
    };
  })
);
